﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Hive.JobBase;
using HeuristicLab.Persistence.Default.Xml;
using System.IO;
using HeuristicLab.Hive.Experiment.Jobs;

namespace HeuristicLab.Hive.Experiment.Views {
  [View("JobItem View")]
  [Content(typeof(JobItem), true)]
  public partial class JobItemView : ItemView {
    public new JobItem Content {
      get { return (JobItem)base.Content; }
      set {
        if (base.Content != value) {
          base.Content = value;
        }
      }
    }

    public JobItemView() {
      InitializeComponent();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content != null) {
        Content_JobChanged(this, EventArgs.Empty);
        Content_ChildJobAdded(this, EventArgs.Empty);
      }
    }

    void Content_JobChanged(object sender, EventArgs e) {
      if (Content.Job != null) {
        logView.Content = Content.Job.Log;
        Content_JobDtoChanged(this, EventArgs.Empty);
        Content_JobStateChanged(this, EventArgs.Empty);
      }
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.JobChanged += new EventHandler(Content_JobChanged);
      Content.JobDtoChanged += new EventHandler(Content_JobDtoChanged);
      Content.JobStateChanged += new EventHandler(Content_JobStateChanged);
      Content.ChildJobAdded += new EventHandler(Content_ChildJobAdded);
    }

    protected override void DeregisterContentEvents() {
      Content.JobChanged -= new EventHandler(Content_JobChanged);
      Content.JobDtoChanged -= new EventHandler(Content_JobDtoChanged);
      Content.JobStateChanged -= new EventHandler(Content_JobStateChanged);
      Content.ChildJobAdded -= new EventHandler(Content_ChildJobAdded);
      base.DeregisterContentEvents();
    }

    private void Content_JobDtoChanged(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(Content_JobDtoChanged), sender, e);
      } else {
        this.jobIdTextBox.Text = Content.JobDto.Id.ToString();
        this.dateCreatedTextBox.Text = Content.JobDto.DateCreated.ToString();
        this.priorityTextBox.Text = Content.JobDto.Priority.ToString();
        this.coresNeededTextBox.Text = Content.JobDto.CoresNeeded.ToString();
        this.memoryNeededTextBox.Text = Content.JobDto.MemoryNeeded.ToString();
        this.projectTextBox.Text = Content.JobDto.Project != null ? Content.JobDto.Project.Name : "-";
      }
    }

    private void Content_JobStateChanged(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(Content_JobStateChanged), sender, e);
      } else {
        this.stateTextBox.Text = Content.JobDto.State.ToString();
        this.percentageTextBox.Text = (Content.JobDto.Percentage * 100).ToString() + "%";
        this.percentageProgressBar.Value = Convert.ToInt32(Content.JobDto.Percentage * percentageProgressBar.Maximum);
        this.dateCalculatedText.Text = Content.JobDto.DateCalculated.ToString();
        this.dateFinishedTextBox.Text = Content.JobDto.DateFinished.ToString();
        this.exceptionTextBox.Text = Content.JobDto.Exception;
      }
    }

    void Content_ChildJobAdded(object sender, EventArgs e) {
      if (InvokeRequired) {
        Invoke(new EventHandler(Content_ChildJobAdded), sender, e);
      } else {
        childJobItemListView.Content = new JobItemList(Content.ChildJobItems);
      }
    }

    protected override void SetEnabledStateOfControls() {
      if (InvokeRequired) {
        this.InvokeIfRequired(c => { SetEnabledStateOfControls(); });
      } else {
        base.SetEnabledStateOfControls();
        this.jobIdTextBox.ReadOnly = this.ReadOnly;
        this.stateTextBox.ReadOnly = this.ReadOnly;
        this.userIdTextBox.ReadOnly = this.ReadOnly;
        this.percentageTextBox.ReadOnly = this.ReadOnly;
        this.dateCreatedTextBox.ReadOnly = this.ReadOnly;
        this.dateCalculatedText.ReadOnly = this.ReadOnly;
        this.dateFinishedTextBox.ReadOnly = this.ReadOnly;
        this.priorityTextBox.ReadOnly = this.ReadOnly;
        this.coresNeededTextBox.ReadOnly = this.ReadOnly;
        this.memoryNeededTextBox.ReadOnly = this.ReadOnly;
        this.exceptionTextBox.ReadOnly = this.ReadOnly;
        this.projectTextBox.ReadOnly = this.ReadOnly;
      }
    }
  }
}
