﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.JobBase;
using HeuristicLab.Optimization;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Experiment.Properties;

namespace HeuristicLab.Hive.Experiment.Jobs {
  [StorableClass]
  public class ExperimentJob : OptimizerJob {
    [Storable]
    private ExecutionState executionState = ExecutionState.Stopped;

    public ExperimentJob() {
      this.ComputeInParallel = true;
    }

    public ExperimentJob(IOptimizer optimizer) : base(optimizer) {
      this.ComputeInParallel = true;
    }

    public override ExecutionState ExecutionState {
      get {
        return ComputeInParallel ? executionState : base.ExecutionState;
      }
    }

    public override void Start() {
      if (ComputeInParallel) {
        executionState = Core.ExecutionState.Started;
        foreach (IJob child in this.ChildJobs) {
          OnNewChildJob(child);
        }
        executionState = Core.ExecutionState.Paused;
        OnWaitForChildJobs();
      } else {
        base.Start();
      }
    }

    public override void Resume(IEnumerable<IJob> childJobs) {
      executionState = Core.ExecutionState.Started;
      this.childJobs = new List<IJob>(childJobs);
      UpdateRuns();
      executionState = Core.ExecutionState.Stopped;
      OnJobStopped();
    }

    private void UpdateRuns() {
      foreach (OptimizerJob optimizerJob in childJobs.Cast<OptimizerJob>()) {
        this.optimizer.Runs.AddRange(optimizerJob.Optimizer.Runs);
      }
    }

    public void AddChildJob(IJob job) {
      this.childJobs.Add(job);
    }

    public override bool IsParallelizable {
      get { return true; }
    }
  }
}
